#include <ignition/msgs.hh>
#include <ignition/transport/Node.hh>

#include <iostream>
#include <cmath>
#include <chrono>
#include <thread>

// Settings
const std::string world_name = "empty";
const std::string model_name = "spawn_robot";

const int timeout_ms = 10;
const double update_rate_hz = 100.0;

// Offsets
double pose_offset_x = 0.0;
double pose_offset_y = 0.0;
double pose_offset_z = 0.17;

int main()
{
    // Connect to the Ignition Transport node
    ignition::transport::Node node;

    // Set service details
    const std::string service = "/world/" + world_name + "/set_pose";
    const std::string reqtype = "ignition.msgs.Pose";
    const std::string reptype = "ignition.msgs.Boolean";

    // Configure the update loop
    auto start_time = std::chrono::steady_clock::now();
    double update_period_s = 1.0 / update_rate_hz;
    auto last_update_time = start_time;

    while (true)
    {
        auto now = std::chrono::steady_clock::now();
        auto time_elapsed = std::chrono::duration_cast<std::chrono::milliseconds>(now - start_time);
        double time_s = time_elapsed.count() / 1000.0;

        if (std::chrono::duration_cast<std::chrono::milliseconds>(now - last_update_time).count() >= update_period_s * 1000.0)
        {
            last_update_time = now;

            // Create request message
            ignition::msgs::Vector3d vector3d_msg;
            pose_offset_x = 10 * sin(time_s);
            pose_offset_y = 10 * cos(time_s);
            pose_offset_z = 0.17;
            vector3d_msg.set_x(pose_offset_x);
            vector3d_msg.set_y(pose_offset_y);
            vector3d_msg.set_z(pose_offset_z);

            ignition::msgs::Quaternion quat_msg;
            // follow circle
            double yaw = atan2(pose_offset_y, pose_offset_x);
            quat_msg.set_w(cos(yaw / 2));
            quat_msg.set_x(0);
            quat_msg.set_y(0);
            quat_msg.set_z(sin(yaw / 2));

            ignition::msgs::Pose pose_msg;
            pose_msg.set_name(model_name);
            pose_msg.mutable_position()->CopyFrom(vector3d_msg);
            pose_msg.mutable_orientation()->CopyFrom(quat_msg);
            bool result;
            ignition::msgs::Boolean result_msg;

            // Submit request (blocking)
            result = node.Request(service, pose_msg, timeout_ms, result_msg, result);

            if (!result)
            {
                std::cout << "[" << time_s << "] Update failed" << std::endl;
            }
        }

        // Sleep for a short duration to avoid high CPU usage
        std::this_thread::sleep_for(std::chrono::milliseconds(10));
    }

    return 0;
}
